@@ -30,6 +30,8 @@ __all__ = [
     "partition_descendants",
     "hadron_vertices",
     "fastjet_clusters",
+    "leaf_masks",
+    "centroid_prune",
 ]
 
 
@@ -46,14 +48,17 @@ def fastjet_clusters(
     eta_cut: ty.Optional[float] = None,
     top_k: ty.Optional[int] = None,
 ) -> ty.List[gcl.MaskArray]:
-    """Clusters particles using the FastJet implementation of the
-    generalised-kt algorithm.
+    """Clusters particles using the **FastJet** implementation of the
+    *generalised-kt algorithm*.
 
     :group: select
 
+    .. versionadded:: 0.2.3
+       Migrated from ``graphicle.calculate.cluster_pmu()``.
+
     Parameters
     ----------
-    pmu: gcl.MomentumArray
+    pmu: MomentumArray
         The momenta of each particle in the point cloud.
     radius : float
         The radius of the clusters to be produced.
@@ -71,7 +76,7 @@ def fastjet_clusters(
         discarded.
     top_k : int, optional
         Only return a maximum ``top_k`` number of jets, sorted by
-        transverse momentum. ie. if ``top_k`` is 3, only 3 jets with
+        transverse momentum. *ie.* if ``top_k`` is 3, only 3 jets with
         highest pt will be given. If ``top_k`` exceeds the number of
         jets reconstructed, all of the jets will be included.
 
@@ -85,11 +90,8 @@ def fastjet_clusters(
     -----
     This is a wrapper around FastJet's implementation.
 
-    ``p_val`` set to ``-1`` gives anti-kT, ``0`` gives Cambridge-Aachen,
-    and ``1`` gives kT clusterings.
-
-    .. versionadded:: 0.2.3
-    Migrated from ``graphicle.calculate.cluster_pmu()``.
+    ``p_val`` set to ``-1`` gives **anti-kT**, ``0`` gives
+    **Cambridge-Aachen**, and ``1`` gives **kT** clusterings.
     """
     pmu_pyjet = pmu.data[["e", "x", "y", "z"]]
     pmu_pyjet.dtype.names = "E", "px", "py", "pz"
@@ -126,6 +128,8 @@ def find_vertex(
 
     :group: select
 
+    .. versionadded:: 0.1.0
+
     Parameters
     ----------
     graph : Graphicle
@@ -195,11 +199,13 @@ def find_vertex(
 
 
 def vertex_descendants(adj: gcl.AdjacencyList, vertex: int) -> gcl.MaskArray:
-    """Return a mask over a graphicle object, identifying which
-    particles descend from a given interaction vertex.
+    """Returns a ``MaskArray`` to select particles which descend from a
+    given interaction vertex.
 
     :group: select
 
+    .. versionadded:: 0.1.0
+
     Parameters
     ----------
     adj : AdjacencyList
@@ -213,10 +219,11 @@ def vertex_descendants(adj: gcl.AdjacencyList, vertex: int) -> gcl.MaskArray:
         Boolean mask over the graphicle objects associated with the
         passed AdjacencyList.
     """
-    bft = breadth_first_tree(adj._sparse, abs(vertex))
-    desc_nodes = (2 * bft.data.astype("<i4") - 1) * bft.indices
-    mask = np.isin(adj.edges["in"], desc_nodes)
-    mask[adj.edges["in"] == vertex] = True
+    sparse = adj._sparse_signed
+    vertex = sparse.row[vertex == adj.edges["in"]][0]
+    bft = breadth_first_tree(sparse, vertex)
+    mask = np.isin(sparse.row, bft.indices)
+    mask[sparse.row == vertex] = True  # include parent vertex
     return gcl.MaskArray(mask)
 
 
@@ -228,6 +235,8 @@ def hadron_vertices(
 
     :group: select
 
+    .. versionadded:: 0.1.11
+
     Parameters
     ----------
     adj : AdjacencyList
@@ -289,7 +298,28 @@ def _partition_vertex(
 
     Parameters
     ----------
-    mask : gcl.MaskArray
+    mask : MaskArray
+        Hard parton descendants.
+    pcls_in : MaskArray or ndarray[bool_]
+        The particles entering the hadronisation vertex.
+    vtx_desc : MaskArray
+        Particles descending from the hadronisation vertex.
+    final : MaskArray
+        Final state particles.
+    pmu : MomentumArray
+        Four momenta.
+    dist_strat : callable
+        Callable which takes two ``MomentumArray`` instances, and
+        returns a double array with number of rows and columns equal to
+        the lengths of the input momenta, respectively. Output should
+        represent pairwise distance between particles incident on the
+        hadronisation vertex, and the final state descendants.
+
+    Returns
+    -------
+    filtered_mask : MaskArray
+        Input ``MaskArray``, filtered to remove background incident on
+        the same hadronisation vertex.
     """
     mask = mask.copy()
     parton_pmu = pmu[pcls_in]
@@ -312,6 +342,8 @@ def partition_descendants(
 
     :group: select
 
+    .. versionadded:: 0.1.11
+
     Parameters
     ----------
     graph : Graphicle
@@ -345,7 +377,12 @@ def partition_descendants(
                 if vtx_id not in graph.edges["out"][mask.data]:
                     continue
                 mask.data = _partition_vertex(
-                    mask, pcls_in, vtx_desc, graph.final, graph.pmu, dist_strat
+                    mask,
+                    pcls_in,
+                    vtx_desc,
+                    graph.final,
+                    graph.pmu,
+                    dist_strat,
                 ).data
     return hier
 
@@ -425,25 +462,31 @@ def hard_descendants(
     target: ty.Optional[ty.Iterable[int]] = None,
     sign_sensitive: bool = False,
 ) -> gcl.MaskGroup:
-    """Returns a MaskGroup over the particles in the graph, where True
-    indicates a particle descends from a specific hard parton.
+    """Returns a ``MaskGroup`` instance to select particle descendants
+    of ``target`` hard partons (by PDG code).
 
     :group: select
 
+    .. versionadded:: 0.1.0
+
+    .. versionchanged:: 0.1.11
+       Target parameter now optional.
+
     Parameters
     ----------
     graph : Graphicle
-        Particle graph containing at least PdgArray and StatusArray.
+        Particle graph containing at least ``PdgArray`` and
+        ``StatusArray``.
     target : Iterable[int], optional
         PDG codes referring to the hard particles for which descendants
         are obtained. If ``None``, will obtain masks for all
         intermediate and outgoing hard partons. Default is ``None``.
     sign_sensitive : bool
         Indicates whether sign of PDG code should be used when selecting
-        hard partons, ie. if set to False both particle and
-        anti-particle partons will be masked, whereas if True only the
-        partons explicitly matching the target sign will be considered.
-        Default is False.
+        hard partons, *ie.* if set to ``False`` both particle and
+        anti-particle partons will be masked, whereas if ``True`` only
+        the partons explicitly matching the target sign will be
+        considered. Default is ``False``.
     """
     hard_vtxs = list()
     # get the vertices of the hard partons
@@ -479,6 +522,8 @@ def hierarchy(
 
     :group: select
 
+    .. versionadded:: 0.1.11
+
     Parameters
     ----------
     graph : Graphicle
@@ -652,6 +697,8 @@ def leaf_masks(mask_tree: gcl.MaskGroup) -> gcl.MaskGroup:
 
     :group: select
 
+    .. versionadded:: 0.1.11
+
     Parameters
     ----------
     mask_tree : MaskGroup
@@ -670,11 +717,13 @@ def leaf_masks(mask_tree: gcl.MaskGroup) -> gcl.MaskGroup:
 
 
 def any_overlap(masks: gcl.MaskGroup) -> bool:
-    """Given a MaskGroup object, checks if any of the masks overlap
+    """Given a ``MaskGroup`` object, checks if any of the masks overlap
     with each other.
 
     :group: select
 
+    .. versionadded:: 0.1.0
+
     Parameters
     ----------
     masks : MaskGroup
@@ -683,10 +732,64 @@ def any_overlap(masks: gcl.MaskGroup) -> bool:
     Returns
     -------
     any_overlap : bool
-        True if at least two MaskArrays in MaskGroup have at least one
-        True element in the same location.
+        ``True`` if at least two ``MaskArrays`` in ``masks`` have at
+        least one ``True`` element in the same location.
     """
     combos = it.combinations(masks.dict.values(), 2)
     pair_checks = map(np.bitwise_and, *zip(*combos))
     overlaps: bool = np.bitwise_or.reduce(tuple(pair_checks), axis=None)
     return overlaps
+
+
+def centroid_prune(
+    pmu: gcl.MomentumArray,
+    radius: float,
+    mask: ty.Optional[gcl.MaskArray] = None,
+    centre: ty.Optional[ty.Tuple[float, float]] = None,
+) -> gcl.MaskArray:
+    """For a given ``MomentumArray``, calculate the distance every
+    particle is from a centroid location, and return a ``MaskArray`` for
+    all of the particles which are within a given ``radius``.
+    If ``centre`` is not provided, the transverse momentum weighted
+    centroid will be used.
+
+    :group: select
+
+    .. versionadded:: 0.2.4
+
+    Parameters
+    ----------
+    pmu : MomentumArray
+        Four-momenta for a set of particles.
+    radius : float
+        Euclidean distance in the azimuth-pseudorapidity plane from the
+        centroid, beyond which particles will be filtered out.
+    mask : MaskArray, optional
+        If provided, will apply the mask to the passed ``pmu``, and
+        output ``MaskArray`` will have the same length.
+    centre : tuple[float, float]
+        Pseudorapidity and azimuth coordinates for a user-defined
+        centroid.
+
+    Returns
+    -------
+    prune_mask : MaskArray
+        Mask which retains only the particles within ``radius`` of the
+        centroid.
+    """
+    if mask is not None:
+        pmu = pmu[mask]
+        event_mask = np.zeros_like(mask, "<?")
+    if centre is None:
+        eta_mid = (pmu.eta * pmu.pt).sum() / pmu.pt.sum()
+        phi_sum_ = (pmu._xy_pol * pmu.pt).sum()
+        phi_mid_ = phi_sum_ / np.abs(phi_sum_)
+    else:
+        eta_mid, phi_mid = centre
+        phi_mid_ = np.exp(complex(0, phi_mid))
+    dist = np.hypot(pmu.eta - eta_mid, np.angle(pmu._xy_pol * phi_mid_.conj()))
+    is_within = dist < radius
+    if mask is None:
+        return gcl.MaskArray(is_within)
+    event_mask[mask] = is_within
+    return gcl.MaskArray(event_mask)
